@@ -22,12 +22,13 @@ enum class lagrange_variant
   gl_isaac = 8,
   gl_centroid = 9,
   legendre = 10,
+  bernstein = 11,
   vtk = 20,
 };
 
-/// Variants of a DPC (discontinuous polynomial cubical) space that can be
-/// created. DPC spaces span the same set of polynomials as Lagrange spaces on
-/// simplices but are defined on tensor product cells.
+/// Variants of a DPC (discontinuous polynomial cubical) space that can
+/// be created. DPC spaces span the same set of polynomials as Lagrange
+/// spaces on simplices but are defined on tensor product cells.
 enum class dpc_variant
 {
   unset = -1,
@@ -55,6 +56,7 @@ enum class family
   bubble = 9,
   serendipity = 10,
   HHJ = 11,
+  Hermite = 12,
+  iso = 13,
 };
-
 } // namespace basix::element
