@@ -1,21 +1,31 @@
 """Helper functions for writing DOLFINx custom kernels using Numba."""
 
-import numpy as _np
-import numpy.typing as _npt
-from typing import List as _ListT
-from typing import Dict as _Dict
 try:
     import numba as _numba
-    from numba.typed import List as _List
-    from numba.core import types as _types
 except ImportError:
     raise RuntimeError("You must have Numba installed to use the Numba helper functions.")
 
+import typing as _typing
+from typing import Dict as _Dict
+from typing import List as _ListT
 
-@_numba.njit
+import numpy as _np
+
+if _typing.TYPE_CHECKING:
+    import numpy.typing as _npt
+    _nda = _npt.NDArray
+    _nda_i32 = _npt.NDArray[_np.int32]
+    _nda_f64 = _npt.NDArray[_np.float64]
+else:
+    _nda = None
+    _nda_i32 = None
+    _nda_f64 = None
+
+
+@_numba.jit(nopython=True)
 def apply_dof_transformation(
-    tdim: int, edge_count: int, face_count: int, entity_transformations: _ListT[int], entity_dofs: _ListT[int],
-    data: _np.array, cell_info: int, face_types: _ListT[str]
+    tdim: int, edge_count: int, face_count: int, entity_transformations: _Dict[str, _nda],
+    entity_dofs: _ListT[_ListT[int]], data: _nda, cell_info: int, face_types: _ListT[str]
 ):
     """Apply dof transformations to some data.
 
@@ -26,8 +36,11 @@ def apply_dof_transformation(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
-        face_types: A list of strings giving the shapes of the faces of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+        face_types: A list of strings giving the shapes of the faces of
+            the cell.
+
     """
     if tdim >= 2:
         if tdim == 3:
@@ -62,10 +75,11 @@ def apply_dof_transformation(
                 dofstart += fdofs
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_interval(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on an interval.
 
@@ -73,15 +87,18 @@ def apply_dof_transformation_interval(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     return
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_triangle(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on a triangle.
 
@@ -89,16 +106,19 @@ def apply_dof_transformation_triangle(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation(2, 3, 1, entity_transformations, entity_dofs,
-                             data, cell_info, _List.empty_list(_types.string))
+                             data, cell_info, _numba.typed.List.empty_list(_numba.core.types.string))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_quadrilateral(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on an quadrilateral.
 
@@ -106,16 +126,19 @@ def apply_dof_transformation_quadrilateral(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation(2, 4, 1, entity_transformations, entity_dofs,
-                             data, cell_info, _List.empty_list(_types.string))
+                             data, cell_info, _numba.typed.List.empty_list(_numba.core.types.string))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_tetrahedron(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on a tetrahedron.
 
@@ -123,16 +146,19 @@ def apply_dof_transformation_tetrahedron(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation(3, 6, 4, entity_transformations, entity_dofs,
-                             data, cell_info, _List(["triangle"] * 4))
+                             data, cell_info, _numba.typed.List(["triangle"] * 4))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_hexahedron(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on a hexahedron.
 
@@ -140,16 +166,19 @@ def apply_dof_transformation_hexahedron(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation(3, 12, 6, entity_transformations, entity_dofs,
-                             data, cell_info, _List(["quadrilateral"] * 6))
+                             data, cell_info, _numba.typed.List(["quadrilateral"] * 6))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_prism(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on an prism.
 
@@ -157,16 +186,19 @@ def apply_dof_transformation_prism(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation(3, 9, 5, entity_transformations, entity_dofs,
-                             data, cell_info, _List(["triangle"] + ["quadrilateral"] * 4 + ["triangle"]))
+                             data, cell_info, _numba.typed.List(["triangle"] + ["quadrilateral"] * 4 + ["triangle"]))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_pyramid(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some data on an prism.
 
@@ -174,16 +206,18 @@ def apply_dof_transformation_pyramid(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation(3, 8, 5, entity_transformations, entity_dofs,
-                             data, cell_info, _List(["quadrilateral"] + ["triangle"] * 4))
+                             data, cell_info, _numba.typed.List(["quadrilateral"] + ["triangle"] * 4))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose(
     tdim: int, edge_count: int, face_count: int, entity_transformations: _ListT[int], entity_dofs: _ListT[int],
-    data: _np.array, cell_info: int, face_types: _ListT[str]
+    data: _nda, cell_info: int, face_types: _ListT[str]
 ):
     """Apply dof transformations to some transposed data.
 
@@ -194,8 +228,11 @@ def apply_dof_transformation_to_transpose(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
-        face_types: A list of strings giving the shapes of the faces of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+        face_types: A list of strings giving the shapes of the faces of
+            the cell.
+
     """
     transposed_data = data.transpose().copy()
     apply_dof_transformation(tdim, edge_count, face_count, entity_transformations, entity_dofs,
@@ -203,10 +240,11 @@ def apply_dof_transformation_to_transpose(
     data[:] = transposed_data.transpose()
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_interval(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on an interval.
 
@@ -214,15 +252,18 @@ def apply_dof_transformation_to_transpose_interval(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     return
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_triangle(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on a triangle.
 
@@ -230,16 +271,19 @@ def apply_dof_transformation_to_transpose_triangle(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation_to_transpose(2, 3, 1, entity_transformations, entity_dofs,
-                                          data, cell_info, _List.empty_list(_types.string))
+                                          data, cell_info, _numba.typed.List.empty_list(_numba.core.types.string))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_quadrilateral(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on an quadrilateral.
 
@@ -250,13 +294,14 @@ def apply_dof_transformation_to_transpose_quadrilateral(
         cell_info: An integer representing the orientations of the subentities of the cell.
     """
     apply_dof_transformation_to_transpose(2, 4, 1, entity_transformations, entity_dofs,
-                                          data, cell_info, _List.empty_list(_types.string))
+                                          data, cell_info, _numba.typed.List.empty_list(_numba.core.types.string))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_tetrahedron(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on a tetrahedron.
 
@@ -267,13 +312,14 @@ def apply_dof_transformation_to_transpose_tetrahedron(
         cell_info: An integer representing the orientations of the subentities of the cell.
     """
     apply_dof_transformation_to_transpose(3, 6, 4, entity_transformations, entity_dofs,
-                                          data, cell_info, _List(["triangle"] * 4))
+                                          data, cell_info, _numba.typed.List(["triangle"] * 4))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_hexahedron(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on a hexahedron.
 
@@ -284,13 +330,14 @@ def apply_dof_transformation_to_transpose_hexahedron(
         cell_info: An integer representing the orientations of the subentities of the cell.
     """
     apply_dof_transformation_to_transpose(3, 12, 6, entity_transformations, entity_dofs,
-                                          data, cell_info, _List(["quadrilateral"] * 6))
+                                          data, cell_info, _numba.typed.List(["quadrilateral"] * 6))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_prism(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on an prism.
 
@@ -298,16 +345,20 @@ def apply_dof_transformation_to_transpose_prism(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
-    apply_dof_transformation_to_transpose(3, 9, 5, entity_transformations, entity_dofs,
-                                          data, cell_info, _List(["triangle"] + ["quadrilateral"] * 4 + ["triangle"]))
+    apply_dof_transformation_to_transpose(
+        3, 9, 5, entity_transformations, entity_dofs,
+        data, cell_info, _numba.typed.List(["triangle"] + ["quadrilateral"] * 4 + ["triangle"]))
 
 
-@_numba.njit
+@_numba.jit(nopython=True)
 def apply_dof_transformation_to_transpose_pyramid(
-    entity_transformations: _Dict[str, _npt.NDArray[_np.float64]], entity_dofs: _Dict[str, _npt.NDArray[_np.int32]],
-    data: _np.array, cell_info: int
+    entity_transformations: _Dict[str, _nda_f64],
+    entity_dofs: _Dict[str, _nda_i32],
+    data: _nda, cell_info: int
 ):
     """Apply dof transformations to some transposed data on an prism.
 
@@ -315,7 +366,9 @@ def apply_dof_transformation_to_transpose_pyramid(
         entity_transformations: The DOF transformations for each entity.
         entity_dofs: The number of DOFs on each entity.
         data: The data. This will be changed by this function.
-        cell_info: An integer representing the orientations of the subentities of the cell.
+        cell_info: An integer representing the orientations of the
+            subentities of the cell.
+
     """
     apply_dof_transformation_to_transpose(3, 8, 5, entity_transformations, entity_dofs,
-                                          data, cell_info, _List(["quadrilateral"] + ["triangle"] * 4))
+                                          data, cell_info, _numba.typed.List(["quadrilateral"] + ["triangle"] * 4))
