@@ -2,28 +2,35 @@
 // FEniCS Project
 // SPDX-License-Identifier:    MIT
 
-#include "docs.h"
 #include <basix/cell.h>
+#include <basix/docs.h>
 #include <basix/element-families.h>
 #include <basix/finite-element.h>
 #include <basix/indexing.h>
 #include <basix/interpolation.h>
 #include <basix/lattice.h>
 #include <basix/maps.h>
+#include <basix/mdspan.hpp>
 #include <basix/polynomials.h>
 #include <basix/polyset.h>
 #include <basix/quadrature.h>
+#include <basix/sobolev-spaces.h>
 #include <pybind11/numpy.h>
 #include <pybind11/operators.h>
 #include <pybind11/pybind11.h>
 #include <pybind11/stl.h>
+#include <span>
 #include <string>
-#include <xtensor/xadapt.hpp>
-#include <xtl/xspan.hpp>
 
 namespace py = pybind11;
 using namespace basix;
 
+namespace stdex = MDSPAN_IMPL_STANDARD_NAMESPACE::MDSPAN_IMPL_PROPOSED_NAMESPACE;
+template <typename T, std::size_t d>
+using mdspan_t
+    = MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<T,
+                    MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, d>>;
+
 namespace
 {
 const std::string& cell_type_to_str(cell::type type)
@@ -45,13 +52,6 @@ const std::string& cell_type_to_str(cell::type type)
   return it->second;
 }
 
-auto adapt_x(const py::array_t<double, py::array::c_style>& x)
-{
-  std::vector<std::size_t> shape;
-  for (pybind11::ssize_t i = 0; i < x.ndim(); ++i)
-    shape.push_back(x.shape(i));
-  return xt::adapt(x.data(), x.size(), xt::no_ownership(), shape);
-}
 } // namespace
 
 PYBIND11_MODULE(_basixcpp, m)
@@ -67,11 +67,8 @@ Interface to the Basix C++ library.
       "geometry",
       [](cell::type celltype)
       {
-        xt::xtensor<double, 2> g = cell::geometry(celltype);
-        auto strides = g.strides();
-        for (auto& s : strides)
-          s *= sizeof(double);
-        return py::array_t<double>(g.shape(), strides, g.data());
+        auto [x, shape] = cell::geometry<double>(celltype);
+        return py::array_t<double>(shape, x.data());
       },
       basix::docstring::geometry.c_str());
   m.def("sub_entity_connectivity", &cell::sub_entity_connectivity,
@@ -80,15 +77,15 @@ Interface to the Basix C++ library.
       "sub_entity_geometry",
       [](cell::type celltype, int dim, int index)
       {
-        xt::xtensor<double, 2> g
-            = cell::sub_entity_geometry(celltype, dim, index);
-        auto strides = g.strides();
-        for (auto& s : strides)
-          s *= sizeof(double);
-        return py::array_t<double>(g.shape(), strides, g.data());
+        auto [x, shape]
+            = cell::sub_entity_geometry<double>(celltype, dim, index);
+        return py::array_t<double>(shape, x.data());
       },
       basix::docstring::sub_entity_geometry.c_str());
 
+  m.def("sobolev_space_intersection", &sobolev::space_intersection,
+        basix::docstring::space_intersection.c_str());
+
   py::enum_<lattice::type>(m, "LatticeType")
       .value("equispaced", lattice::type::equispaced)
       .value("gll", lattice::type::gll)
@@ -102,7 +99,8 @@ Interface to the Basix C++ library.
       .value("centroid", lattice::simplex_method::centroid);
 
   py::enum_<polynomials::type>(m, "PolynomialType")
-      .value("legendre", polynomials::type::legendre);
+      .value("legendre", polynomials::type::legendre)
+      .value("bernstein", polynomials::type::bernstein);
 
   m.def(
       "tabulate_polynomials",
@@ -111,25 +109,23 @@ Interface to the Basix C++ library.
       {
         if (x.ndim() != 2)
           throw std::runtime_error("x has the wrong number of dimensions");
-        std::array<std::size_t, 2> shape
-            = {(std::size_t)x.shape(0), (std::size_t)x.shape(1)};
-        auto _x = xt::adapt(x.data(), x.size(), xt::no_ownership(), shape);
-        xt::xtensor<double, 2> t
-            = polynomials::tabulate(polytype, celltype, d, _x);
-        return py::array_t<double>(t.shape(), t.data());
+        MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<const double,
+                      MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
+            _x(x.data(), x.shape(0), x.shape(1));
+        auto [p, shape] = polynomials::tabulate(polytype, celltype, d, _x);
+        return py::array_t<double>(shape, p.data());
       },
       basix::docstring::tabulate_polynomials.c_str());
+  m.def("polynomials_dim", &polynomials::dim,
+        basix::docstring::polynomials_dim.c_str());
 
   m.def(
       "create_lattice",
       [](cell::type celltype, int n, lattice::type type, bool exterior)
       {
-        auto l = lattice::create(celltype, n, type, exterior,
-                                 lattice::simplex_method::none);
-        auto strides = l.strides();
-        for (auto& s : strides)
-          s *= sizeof(double);
-        return py::array_t<double>(l.shape(), strides, l.data());
+        auto [x, shape] = lattice::create<double>(
+            celltype, n, type, exterior, lattice::simplex_method::none);
+        return py::array_t<double>(shape, x.data());
       },
       basix::docstring::create_lattice__celltype_n_type_exterior.c_str());
 
@@ -138,11 +134,9 @@ Interface to the Basix C++ library.
       [](cell::type celltype, int n, lattice::type type, bool exterior,
          lattice::simplex_method method)
       {
-        auto l = lattice::create(celltype, n, type, exterior, method);
-        auto strides = l.strides();
-        for (auto& s : strides)
-          s *= sizeof(double);
-        return py::array_t<double>(l.shape(), strides, l.data());
+        auto [x, shape]
+            = lattice::create<double>(celltype, n, type, exterior, method);
+        return py::array_t<double>(shape, x.data());
       },
       basix::docstring::create_lattice__celltype_n_type_exterior_method
           .c_str());
@@ -155,6 +149,17 @@ Interface to the Basix C++ library.
       .value("doubleCovariantPiola", maps::type::doubleCovariantPiola)
       .value("doubleContravariantPiola", maps::type::doubleContravariantPiola);
 
+  py::enum_<sobolev::space>(m, "SobolevSpace")
+      .value("L2", sobolev::space::L2)
+      .value("H1", sobolev::space::H1)
+      .value("H2", sobolev::space::H2)
+      .value("H3", sobolev::space::H3)
+      .value("HInf", sobolev::space::HInf)
+      .value("HDiv", sobolev::space::HDiv)
+      .value("HCurl", sobolev::space::HCurl)
+      .value("HEin", sobolev::space::HEin)
+      .value("HDivDiv", sobolev::space::HDivDiv);
+
   py::enum_<quadrature::type>(m, "QuadratureType")
       .value("Default", quadrature::type::Default)
       .value("gauss_jacobi", quadrature::type::gauss_jacobi)
@@ -173,31 +178,33 @@ Interface to the Basix C++ library.
 
   m.def(
       "cell_volume",
-      [](cell::type cell_type) -> double { return cell::volume(cell_type); },
+      [](cell::type cell_type) -> double
+      { return cell::volume<double>(cell_type); },
       basix::docstring::cell_volume.c_str());
   m.def(
       "cell_facet_normals",
       [](cell::type cell_type)
       {
-        xt::xtensor<double, 2> normals = cell::facet_normals(cell_type);
-        return py::array_t<double>(normals.shape(), normals.data());
+        auto [n, shape] = cell::facet_normals<double>(cell_type);
+        return py::array_t<double>(shape, n.data());
       },
       basix::docstring::cell_facet_normals.c_str());
   m.def(
       "cell_facet_reference_volumes",
       [](cell::type cell_type)
       {
-        xt::xtensor<double, 1> volumes
-            = cell::facet_reference_volumes(cell_type);
-        return py::array_t<double>(volumes.shape(), volumes.data());
+        std::vector<double> v
+            = cell::facet_reference_volumes<double>(cell_type);
+        std::array<std::size_t, 1> shape = {v.size()};
+        return py::array_t<double>(shape, v.data());
       },
       basix::docstring::cell_facet_reference_volumes.c_str());
   m.def(
       "cell_facet_outward_normals",
       [](cell::type cell_type)
       {
-        xt::xtensor<double, 2> normals = cell::facet_outward_normals(cell_type);
-        return py::array_t<double>(normals.shape(), normals.data());
+        auto [n, shape] = cell::facet_outward_normals<double>(cell_type);
+        return py::array_t<double>(shape, n.data());
       },
       basix::docstring::cell_facet_outward_normals.c_str());
   m.def("cell_facet_orientations", &cell::facet_orientations,
@@ -206,8 +213,8 @@ Interface to the Basix C++ library.
       "cell_facet_jacobians",
       [](cell::type cell_type)
       {
-        xt::xtensor<double, 3> jacobians = cell::facet_jacobians(cell_type);
-        return py::array_t<double>(jacobians.shape(), jacobians.data());
+        auto [J, shape] = cell::facet_jacobians<double>(cell_type);
+        return py::array_t<double>(shape, J.data());
       },
       basix::docstring::cell_facet_jacobians.c_str());
 
@@ -223,68 +230,81 @@ Interface to the Basix C++ library.
       .value("bubble", element::family::bubble)
       .value("serendipity", element::family::serendipity)
       .value("DPC", element::family::DPC)
-      .value("CR", element::family::CR);
+      .value("CR", element::family::CR)
+      .value("Hermite", element::family::Hermite)
+      .value("iso", element::family::iso);
 
-  py::class_<FiniteElement>(m, "FiniteElement", "Finite Element")
+  py::class_<FiniteElement<double>>(m, "FiniteElement", "Finite Element")
       .def(
           "tabulate",
-          [](const FiniteElement& self, int n,
+          [](const FiniteElement<double>& self, int n,
              const py::array_t<double, py::array::c_style>& x)
           {
             if (x.ndim() != 2)
               throw std::runtime_error("x has the wrong size");
-            std::array<std::size_t, 2> shape
-                = {(std::size_t)x.shape(0), (std::size_t)x.shape(1)};
-            auto _x = xt::adapt(x.data(), x.size(), xt::no_ownership(), shape);
-            auto t = self.tabulate(n, _x);
-            return py::array_t<double>(t.shape(), t.data());
+            MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<
+                const double,
+                MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
+                _x(x.data(), x.shape(0), x.shape(1));
+            auto [t, shape] = self.tabulate(n, _x);
+            return py::array_t<double>(shape, t.data());
           },
           basix::docstring::FiniteElement__tabulate.c_str())
-      .def("__eq__", &FiniteElement::operator==)
+      .def("__eq__", &FiniteElement<double>::operator==)
       .def(
           "push_forward",
-          [](const FiniteElement& self,
+          [](const FiniteElement<double>& self,
              const py::array_t<double, py::array::c_style>& U,
              const py::array_t<double, py::array::c_style>& J,
              const py::array_t<double, py::array::c_style>& detJ,
              const py::array_t<double, py::array::c_style>& K)
           {
-            auto u = self.push_forward(
-                adapt_x(U), adapt_x(J),
-                xtl::span<const double>(detJ.data(), detJ.size()), adapt_x(K));
-            return py::array_t<double>(u.shape(), u.data());
+            auto [u, shape] = self.push_forward(
+                mdspan_t<const double, 3>(U.data(), U.shape(0), U.shape(1),
+                                          U.shape(2)),
+                mdspan_t<const double, 3>(J.data(), J.shape(0), J.shape(1),
+                                          J.shape(2)),
+                std::span<const double>(detJ.data(), detJ.size()),
+                mdspan_t<const double, 3>(K.data(), K.shape(0), K.shape(1),
+                                          K.shape(2)));
+            return py::array_t<double>(shape, u.data());
           },
           basix::docstring::FiniteElement__push_forward.c_str())
       .def(
           "pull_back",
-          [](const FiniteElement& self,
+          [](const FiniteElement<double>& self,
              const py::array_t<double, py::array::c_style>& u,
              const py::array_t<double, py::array::c_style>& J,
              const py::array_t<double, py::array::c_style>& detJ,
              const py::array_t<double, py::array::c_style>& K)
           {
-            auto U = self.pull_back(
-                adapt_x(u), adapt_x(J),
-                xtl::span<const double>(detJ.data(), detJ.size()), adapt_x(K));
-            return py::array_t<double>(U.shape(), U.data());
+            auto [U, shape] = self.pull_back(
+                mdspan_t<const double, 3>(u.data(), u.shape(0), u.shape(1),
+                                          u.shape(2)),
+                mdspan_t<const double, 3>(J.data(), J.shape(0), J.shape(1),
+                                          J.shape(2)),
+                std::span<const double>(detJ.data(), detJ.size()),
+                mdspan_t<const double, 3>(K.data(), K.shape(0), K.shape(1),
+                                          K.shape(2)));
+            return py::array_t<double>(shape, U.data());
           },
           basix::docstring::FiniteElement__pull_back.c_str())
       .def(
           "apply_dof_transformation",
-          [](const FiniteElement& self, py::array_t<double>& data,
+          [](const FiniteElement<double>& self, py::array_t<double>& data,
              int block_size, std::uint32_t cell_info)
           {
-            xtl::span<double> data_span(data.mutable_data(), data.size());
+            std::span<double> data_span(data.mutable_data(), data.size());
             self.apply_dof_transformation(data_span, block_size, cell_info);
             return py::array_t<double>(data_span.size(), data_span.data());
           },
           basix::docstring::FiniteElement__apply_dof_transformation.c_str())
       .def(
           "apply_dof_transformation_to_transpose",
-          [](const FiniteElement& self, py::array_t<double>& data,
+          [](const FiniteElement<double>& self, py::array_t<double>& data,
              int block_size, std::uint32_t cell_info)
           {
-            xtl::span<double> data_span(data.mutable_data(), data.size());
+            std::span<double> data_span(data.mutable_data(), data.size());
             self.apply_dof_transformation_to_transpose(data_span, block_size,
                                                        cell_info);
             return py::array_t<double>(data_span.size(), data_span.data());
@@ -293,10 +313,10 @@ Interface to the Basix C++ library.
               .c_str())
       .def(
           "apply_inverse_transpose_dof_transformation",
-          [](const FiniteElement& self, py::array_t<double>& data,
+          [](const FiniteElement<double>& self, py::array_t<double>& data,
              int block_size, std::uint32_t cell_info)
           {
-            xtl::span<double> data_span(data.mutable_data(), data.size());
+            std::span<double> data_span(data.mutable_data(), data.size());
             self.apply_inverse_transpose_dof_transformation(
                 data_span, block_size, cell_info);
             return py::array_t<double>(data_span.size(), data_span.data());
@@ -305,107 +325,142 @@ Interface to the Basix C++ library.
               FiniteElement__apply_inverse_transpose_dof_transformation.c_str())
       .def(
           "base_transformations",
-          [](const FiniteElement& self)
+          [](const FiniteElement<double>& self)
           {
-            xt::xtensor<double, 3> t = self.base_transformations();
-            return py::array_t<double>(t.shape(), t.data());
+            auto [t, shape] = self.base_transformations();
+            return py::array_t<double>(shape, t.data());
           },
           basix::docstring::FiniteElement__base_transformations.c_str())
       .def(
           "entity_transformations",
-          [](const FiniteElement& self)
+          [](const FiniteElement<double>& self)
           {
-            std::map<cell::type, xt::xtensor<double, 3>> t
-                = self.entity_transformations();
+            auto t = self.entity_transformations();
             py::dict t2;
-            for (auto tpart : t)
+            for (auto& [key, data] : t)
             {
-              t2[cell_type_to_str(tpart.first).c_str()] = py::array_t<double>(
-                  tpart.second.shape(), tpart.second.data());
+              t2[cell_type_to_str(key).c_str()]
+                  = py::array_t<double>(data.second, data.first.data());
             }
             return t2;
           },
           basix::docstring::FiniteElement__entity_transformations.c_str())
       .def(
           "get_tensor_product_representation",
-          [](const FiniteElement& self)
+          [](const FiniteElement<double>& self)
           { return self.get_tensor_product_representation(); },
           basix::docstring::FiniteElement__get_tensor_product_representation
               .c_str())
-      .def_property_readonly("degree", &FiniteElement::degree)
-      .def_property_readonly("highest_degree", &FiniteElement::highest_degree)
+      .def_property_readonly("degree", &FiniteElement<double>::degree)
+      .def_property_readonly("highest_degree",
+                             &FiniteElement<double>::highest_degree)
       .def_property_readonly("highest_complete_degree",
-                             &FiniteElement::highest_complete_degree)
-      .def_property_readonly("cell_type", &FiniteElement::cell_type)
-      .def_property_readonly("dim", &FiniteElement::dim)
-      .def_property_readonly("num_entity_dofs", &FiniteElement::num_entity_dofs)
-      .def_property_readonly("entity_dofs", &FiniteElement::entity_dofs)
+                             &FiniteElement<double>::highest_complete_degree)
+      .def_property_readonly("cell_type", &FiniteElement<double>::cell_type)
+      .def_property_readonly("polyset_type",
+                             &FiniteElement<double>::polyset_type)
+      .def_property_readonly("dim", &FiniteElement<double>::dim)
+      .def_property_readonly("num_entity_dofs",
+                             [](const FiniteElement<double>& self)
+                             {
+                               // TODO: remove this function. Information can
+                               // retrieved from entity_dofs.
+                               auto& edofs = self.entity_dofs();
+                               std::vector<std::vector<int>> num_edofs;
+                               for (auto& edofs_d : edofs)
+                               {
+                                 auto& ndofs = num_edofs.emplace_back();
+                                 for (auto& edofs : edofs_d)
+                                   ndofs.push_back(edofs.size());
+                               }
+                               return num_edofs;
+                             })
+      .def_property_readonly("entity_dofs", &FiniteElement<double>::entity_dofs)
       .def_property_readonly("num_entity_closure_dofs",
-                             &FiniteElement::num_entity_closure_dofs)
+                             [](const FiniteElement<double>& self)
+                             {
+                               // TODO: remove this function. Information can
+                               // retrieved from entity_closure_dofs.
+                               auto& edofs = self.entity_closure_dofs();
+                               std::vector<std::vector<int>> num_edofs;
+                               for (auto& edofs_d : edofs)
+                               {
+                                 auto& ndofs = num_edofs.emplace_back();
+                                 for (auto& edofs : edofs_d)
+                                   ndofs.push_back(edofs.size());
+                               }
+                               return num_edofs;
+                             })
       .def_property_readonly("entity_closure_dofs",
-                             &FiniteElement::entity_closure_dofs)
+                             &FiniteElement<double>::entity_closure_dofs)
       .def_property_readonly("value_size",
-                             [](const FiniteElement& self)
+                             [](const FiniteElement<double>& self)
                              {
                                return std::accumulate(
                                    self.value_shape().begin(),
                                    self.value_shape().end(), 1,
-                                   std::multiplies<int>());
+                                   std::multiplies{});
                              })
-      .def_property_readonly("value_shape", &FiniteElement::value_shape)
-      .def_property_readonly("discontinuous", &FiniteElement::discontinuous)
-      .def_property_readonly("family", &FiniteElement::family)
+      .def_property_readonly("value_shape", &FiniteElement<double>::value_shape)
+      .def_property_readonly("discontinuous",
+                             &FiniteElement<double>::discontinuous)
+      .def_property_readonly("family", &FiniteElement<double>::family)
       .def_property_readonly("lagrange_variant",
-                             &FiniteElement::lagrange_variant)
-      .def_property_readonly("dpc_variant", &FiniteElement::dpc_variant)
+                             &FiniteElement<double>::lagrange_variant)
+      .def_property_readonly("dpc_variant", &FiniteElement<double>::dpc_variant)
       .def_property_readonly(
           "dof_transformations_are_permutations",
-          &FiniteElement::dof_transformations_are_permutations)
-      .def_property_readonly("dof_transformations_are_identity",
-                             &FiniteElement::dof_transformations_are_identity)
+          &FiniteElement<double>::dof_transformations_are_permutations)
+      .def_property_readonly(
+          "dof_transformations_are_identity",
+          &FiniteElement<double>::dof_transformations_are_identity)
       .def_property_readonly("interpolation_is_identity",
-                             &FiniteElement::interpolation_is_identity)
-      .def_property_readonly("map_type", &FiniteElement::map_type)
+                             &FiniteElement<double>::interpolation_is_identity)
+      .def_property_readonly("map_type", &FiniteElement<double>::map_type)
+      .def_property_readonly("sobolev_space",
+                             &FiniteElement<double>::sobolev_space)
       .def_property_readonly("points",
-                             [](const FiniteElement& self)
+                             [](const FiniteElement<double>& self)
                              {
-                               const xt::xtensor<double, 2>& x = self.points();
-                               return py::array_t<double>(x.shape(), x.data(),
+                               auto& [x, shape] = self.points();
+                               return py::array_t<double>(shape, x.data(),
                                                           py::cast(self));
                              })
-      .def_property_readonly(
-          "interpolation_matrix",
-          [](const FiniteElement& self)
-          {
-            const xt::xtensor<double, 2>& P = self.interpolation_matrix();
-            return py::array_t<double>(P.shape(), P.data(), py::cast(self));
-          })
-      .def_property_readonly(
-          "dual_matrix",
-          [](const FiniteElement& self)
-          {
-            const xt::xtensor<double, 2>& P = self.dual_matrix();
-            return py::array_t<double>(P.shape(), P.data(), py::cast(self));
-          })
-      .def_property_readonly(
-          "coefficient_matrix",
-          [](const FiniteElement& self)
-          {
-            const xt::xtensor<double, 2>& P = self.coefficient_matrix();
-            return py::array_t<double>(P.shape(), P.data(), py::cast(self));
-          })
-      .def_property_readonly("wcoeffs",
-                             [](const FiniteElement& self)
+      .def_property_readonly("interpolation_matrix",
+                             [](const FiniteElement<double>& self)
                              {
-                               const xt::xtensor<double, 2>& P = self.wcoeffs();
-                               return py::array_t<double>(P.shape(), P.data(),
+                               auto& [P, shape] = self.interpolation_matrix();
+                               return py::array_t<double>(shape, P.data(),
+                                                          py::cast(self));
+                             })
+      .def_property_readonly("dual_matrix",
+                             [](const FiniteElement<double>& self)
+                             {
+                               auto& [D, shape] = self.dual_matrix();
+                               return py::array_t<double>(shape, D.data(),
+                                                          py::cast(self));
+                             })
+      .def_property_readonly("coefficient_matrix",
+                             [](const FiniteElement<double>& self)
+                             {
+                               auto& [P, shape] = self.coefficient_matrix();
+                               return py::array_t<double>(shape, P.data(),
+                                                          py::cast(self));
+                             })
+      .def_property_readonly("wcoeffs",
+                             [](const FiniteElement<double>& self)
+                             {
+                               auto& [P, shape] = self.wcoeffs();
+                               return py::array_t<double>(shape, P.data(),
                                                           py::cast(self));
                              })
       .def_property_readonly(
           "M",
-          [](const FiniteElement& self)
+          [](const FiniteElement<double>& self)
           {
-            const std::array<std::vector<xt::xtensor<double, 3>>, 4>& _M
+            const std::array<std::vector<std::pair<std::vector<double>,
+                                                   std::array<std::size_t, 4>>>,
+                             4>& _M
                 = self.M();
             std::vector<std::vector<py::array_t<double, py::array::c_style>>> M(
                 4);
@@ -413,17 +468,20 @@ Interface to the Basix C++ library.
             {
               for (std::size_t j = 0; j < _M[i].size(); ++j)
               {
-                M[i].push_back(py::array_t<double>(
-                    _M[i][j].shape(), _M[i][j].data(), py::cast(self)));
+                auto& mat = _M[i][j];
+                M[i].push_back(py::array_t<double>(mat.second, mat.first.data(),
+                                                   py::cast(self)));
               }
             }
             return M;
           })
       .def_property_readonly(
           "x",
-          [](const FiniteElement& self)
+          [](const FiniteElement<double>& self)
           {
-            const std::array<std::vector<xt::xtensor<double, 2>>, 4>& _x
+            const std::array<std::vector<std::pair<std::vector<double>,
+                                                   std::array<std::size_t, 2>>>,
+                             4>& _x
                 = self.x();
             std::vector<std::vector<py::array_t<double, py::array::c_style>>> x(
                 4);
@@ -431,14 +489,20 @@ Interface to the Basix C++ library.
             {
               for (std::size_t j = 0; j < _x[i].size(); ++j)
               {
-                x[i].push_back(py::array_t<double>(
-                    _x[i][j].shape(), _x[i][j].data(), py::cast(self)));
+                auto& vec = _x[i][j];
+                x[i].push_back(py::array_t<double>(vec.second, vec.first.data(),
+                                                   py::cast(self)));
               }
             }
             return x;
           })
-      .def_property_readonly("has_tensor_product_factorisation",
-                             &FiniteElement::has_tensor_product_factorisation);
+      .def_property_readonly(
+          "has_tensor_product_factorisation",
+          &FiniteElement<double>::has_tensor_product_factorisation)
+      .def_property_readonly("interpolation_nderivs",
+                             &FiniteElement<double>::interpolation_nderivs)
+      .def_property_readonly("dof_ordering",
+                             &FiniteElement<double>::dof_ordering);
 
   py::enum_<element::lagrange_variant>(m, "LagrangeVariant")
       .value("unset", element::lagrange_variant::unset)
@@ -454,6 +518,7 @@ Interface to the Basix C++ library.
       .value("gl_isaac", element::lagrange_variant::gl_isaac)
       .value("gl_centroid", element::lagrange_variant::gl_centroid)
       .value("legendre", element::lagrange_variant::legendre)
+      .value("bernstein", element::lagrange_variant::bernstein)
       .value("vtk", element::lagrange_variant::vtk);
 
   py::enum_<element::dpc_variant>(m, "DPCVariant")
@@ -476,34 +541,38 @@ Interface to the Basix C++ library.
              std::vector<py::array_t<double, py::array::c_style>>>& x,
          const std::vector<
              std::vector<py::array_t<double, py::array::c_style>>>& M,
-         maps::type map_type, bool discontinuous, int highest_complete_degree,
-         int highest_degree) -> FiniteElement {
+         int interpolation_nderivs, maps::type map_type,
+         sobolev::space sobolev_space, bool discontinuous,
+         int highest_complete_degree, int highest_degree,
+         polyset::type poly_type) -> FiniteElement<double>
+      {
         if (x.size() != 4)
           throw std::runtime_error("x has the wrong size");
         if (M.size() != 4)
           throw std::runtime_error("M has the wrong size");
 
-        xt::xtensor<double, 2> _wco = adapt_x(wcoeffs);
-
-        std::array<std::vector<xt::xtensor<double, 2>>, 4> _x;
+        std::array<std::vector<impl::mdspan_t<const double, 2>>, 4> _x;
         for (int i = 0; i < 4; ++i)
         {
           for (std::size_t j = 0; j < x[i].size(); ++j)
           {
             if (x[i][j].ndim() != 2)
               throw std::runtime_error("x has the wrong number of dimensions");
-            _x[i].push_back(adapt_x(x[i][j]));
+            _x[i].emplace_back(x[i][j].data(), x[i][j].shape(0),
+                               x[i][j].shape(1));
           }
         }
 
-        std::array<std::vector<xt::xtensor<double, 3>>, 4> _M;
+        std::array<std::vector<impl::mdspan_t<const double, 4>>, 4> _M;
         for (int i = 0; i < 4; ++i)
         {
           for (std::size_t j = 0; j < M[i].size(); ++j)
           {
-            if (M[i][j].ndim() != 3)
+            if (M[i][j].ndim() != 4)
               throw std::runtime_error("M has the wrong number of dimensions");
-            _M[i].push_back(adapt_x(M[i][j]));
+            _M[i].emplace_back(M[i][j].data(), M[i][j].shape(0),
+                               M[i][j].shape(1), M[i][j].shape(2),
+                               M[i][j].shape(3));
           }
         }
 
@@ -511,140 +580,91 @@ Interface to the Basix C++ library.
         for (std::size_t i = 0; i < value_shape.size(); ++i)
           _vs[i] = static_cast<std::size_t>(value_shape[i]);
 
-        return basix::create_custom_element(
-            cell_type, _vs, _wco, _x, _M, map_type, discontinuous,
-            highest_complete_degree, highest_degree);
+        return basix::create_custom_element<double>(
+            cell_type, _vs,
+            mdspan_t<const double, 2>(wcoeffs.data(), wcoeffs.shape(0),
+                                      wcoeffs.shape(1)),
+            _x, _M, interpolation_nderivs, map_type, sobolev_space,
+            discontinuous, highest_complete_degree, highest_degree, poly_type);
       },
       basix::docstring::create_custom_element.c_str());
 
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name, int degree,
-         bool discontinuous) -> FiniteElement
-      {
-        return basix::create_element(family_name, cell_name, degree,
-                                     discontinuous);
-      },
-      basix::docstring::create_element__family_cell_degree_discontinuous
-          .c_str());
-
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name, int degree,
-         element::lagrange_variant lvariant,
-         bool discontinuous) -> FiniteElement
-      {
-        return basix::create_element(family_name, cell_name, degree, lvariant,
-                                     discontinuous);
-      },
-      basix::docstring::
-          create_element__family_cell_degree_lvariant_discontinuous.c_str());
-
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name, int degree,
-         element::dpc_variant dvariant, bool discontinuous) -> FiniteElement
-      {
-        return basix::create_element(family_name, cell_name, degree, dvariant,
-                                     discontinuous);
-      },
-      basix::docstring::
-          create_element__family_cell_degree_dvariant_discontinuous.c_str());
-
   m.def(
       "create_element",
       [](element::family family_name, cell::type cell_name, int degree,
          element::lagrange_variant lvariant, element::dpc_variant dvariant,
-         bool discontinuous) -> FiniteElement
+         bool discontinuous,
+         const std::vector<int>& dof_ordering) -> FiniteElement<double>
       {
-        return basix::create_element(family_name, cell_name, degree, lvariant,
-                                     dvariant, discontinuous);
+        return basix::create_element<double>(family_name, cell_name, degree,
+                                             lvariant, dvariant, discontinuous,
+                                             dof_ordering);
       },
+      py::arg("family_name"), py::arg("cell_name"), py::arg("degree"),
+      py::arg("lagrange_variant") = element::lagrange_variant::unset,
+      py::arg("dpc_variant") = element::dpc_variant::unset,
+      py::arg("discontinuous") = false,
+      py::arg("dof_ordering") = std::vector<int>(),
       basix::docstring::
-          create_element__family_cell_degree_lvariant_dvariant_discontinuous
+          create_element__family_cell_degree_lvariant_dvariant_discontinuous_dof_ordering
               .c_str());
 
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name,
-         int degree) -> FiniteElement
-      { return basix::create_element(family_name, cell_name, degree); },
-      basix::docstring::create_element__family_cell_degree.c_str());
-
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name, int degree,
-         element::lagrange_variant lvariant) -> FiniteElement {
-        return basix::create_element(family_name, cell_name, degree, lvariant);
-      },
-      basix::docstring::create_element__family_cell_degree_lvariant.c_str());
-
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name, int degree,
-         element::dpc_variant dvariant) -> FiniteElement {
-        return basix::create_element(family_name, cell_name, degree, dvariant);
-      },
-      basix::docstring::create_element__family_cell_degree_dvariant.c_str());
-
-  m.def(
-      "create_element",
-      [](element::family family_name, cell::type cell_name, int degree,
-         element::lagrange_variant lvariant,
-         element::dpc_variant dvariant) -> FiniteElement
-      {
-        return basix::create_element(family_name, cell_name, degree, lvariant,
-                                     dvariant);
-      },
-      basix::docstring::create_element__family_cell_degree_lvariant_dvariant
-          .c_str());
-
   // Interpolate between elements
   m.def(
       "compute_interpolation_operator",
-      [](const FiniteElement& element_from, const FiniteElement& element_to)
+      [](const FiniteElement<double>& element_from,
+         const FiniteElement<double>& element_to)
           -> const py::array_t<double, py::array::c_style>
       {
-        xt::xtensor<double, 2> out
-            = basix::compute_interpolation_operator(element_from, element_to);
-        return py::array_t<double>(out.shape(), out.data());
+        auto [out, shape] = basix::compute_interpolation_operator<double>(
+            element_from, element_to);
+        return py::array_t<double>(shape, out.data());
       },
       basix::docstring::compute_interpolation_operator.c_str());
 
+  py::enum_<polyset::type>(m, "PolysetType")
+      .value("standard", polyset::type::standard)
+      .value("macroedge", polyset::type::macroedge);
+
+  m.def(
+      "superset",
+      [](cell::type cell, polyset::type type1, polyset::type type2)
+      { return polyset::superset(cell, type1, type2); },
+      basix::docstring::superset.c_str());
+
+  m.def(
+      "restriction",
+      [](polyset::type ptype, cell::type cell, cell::type restriction_cell)
+      { return polyset::restriction(ptype, cell, restriction_cell); },
+      basix::docstring::restriction.c_str());
+
   m.def(
       "tabulate_polynomial_set",
-      [](cell::type celltype, int d, int n,
+      [](cell::type celltype, polyset::type polytype, int d, int n,
          const py::array_t<double, py::array::c_style>& x)
       {
         if (x.ndim() != 2)
           throw std::runtime_error("x has the wrong number of dimensions");
-        std::array<std::size_t, 2> shape
-            = {(std::size_t)x.shape(0), (std::size_t)x.shape(1)};
-        auto _x = xt::adapt(x.data(), x.size(), xt::no_ownership(), shape);
-        xt::xtensor<double, 3> P = polyset::tabulate(celltype, d, n, _x);
-        return py::array_t<double>(P.shape(), P.data());
+        MDSPAN_IMPL_STANDARD_NAMESPACE::mdspan<const double,
+                      MDSPAN_IMPL_STANDARD_NAMESPACE::dextents<std::size_t, 2>>
+            _x(x.data(), x.shape(0), x.shape(1));
+        auto [p, shape] = polyset::tabulate(celltype, polytype, d, n, _x);
+        return py::array_t<double>(shape, p.data());
       },
       basix::docstring::tabulate_polynomial_set.c_str());
 
   m.def(
       "make_quadrature",
-      [](quadrature::type rule, cell::type celltype, int m)
+      [](quadrature::type rule, cell::type celltype, polyset::type polytype,
+         int m)
       {
-        auto [pts, w] = quadrature::make_quadrature(rule, celltype, m);
-        return std::pair(py::array_t<double>(pts.shape(), pts.data()),
+        auto [pts, w]
+            = quadrature::make_quadrature<double>(rule, celltype, polytype, m);
+        std::array<std::size_t, 2> shape = {w.size(), pts.size() / w.size()};
+        return std::pair(py::array_t<double>(shape, pts.data()),
                          py::array_t<double>(w.size(), w.data()));
       },
-      basix::docstring::make_quadrature__rule_celltype_m.c_str());
-
-  m.def(
-      "make_quadrature",
-      [](cell::type celltype, int m)
-      {
-        auto [pts, w] = quadrature::make_quadrature(celltype, m);
-        return std::pair(py::array_t<double>(pts.shape(), pts.data()),
-                         py::array_t<double>(w.size(), w.data()));
-      },
-      basix::docstring::make_quadrature__celltype_m.c_str());
+      basix::docstring::make_quadrature__rule_celltype_polytype_m.c_str());
 
   m.def("index", py::overload_cast<int>(&basix::indexing::idx),
         basix::docstring::index__p.c_str())
