@@ -2,9 +2,12 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
 import pytest
 
+import basix
+
+from .utils import parametrize_over_elements
+
 cells = [
     basix.CellType.point,
     basix.CellType.interval,
@@ -28,6 +31,8 @@ elements = [
     basix.ElementFamily.serendipity,
     basix.ElementFamily.DPC,
     basix.ElementFamily.CR,
+    basix.ElementFamily.Hermite,
+    basix.ElementFamily.iso,
     basix.ElementFamily.custom,
 ]
 
@@ -35,8 +40,9 @@ variants = [
     [basix.LagrangeVariant.gll_isaac],
     [basix.LagrangeVariant.gll_warped],
     [basix.LagrangeVariant.legendre],
-    [basix.DPCVariant.diagonal_gll],
-    [basix.DPCVariant.legendre],
+    [basix.LagrangeVariant.bernstein],
+    [basix.LagrangeVariant.unset, basix.DPCVariant.diagonal_gll],
+    [basix.LagrangeVariant.unset, basix.DPCVariant.legendre],
     [basix.LagrangeVariant.legendre, basix.DPCVariant.legendre],
 ]
 
@@ -59,6 +65,27 @@ def test_all_elements_included():
     assert all_elements == set(elements)
 
 
+def test_all_cells_included_in_paramerize():
+    all_cells = set()
+    for c in dir(basix.CellType):
+        if not c.startswith("_") and c not in ["name", "value", "point"]:
+            all_cells.add(getattr(basix.CellType, c))
+
+    assert all_cells == set(i[0] for i in parametrize_over_elements(4).mark.args[1])
+
+
+def test_all_elements_included_in_parametrize():
+    all_elements = set()
+    for c in dir(basix.ElementFamily):
+        if not c.startswith("_") and c not in ["name", "value", "custom"]:
+            all_elements.add(getattr(basix.ElementFamily, c))
+
+    assert all_elements == set([
+        i[1] for i in parametrize_over_elements(4).mark.args[1]
+    ] + [
+        i[1] for i in parametrize_over_elements(4, discontinuous=True).mark.args[1]])
+
+
 @pytest.mark.parametrize("cell", cells)
 @pytest.mark.parametrize("degree", range(-1, 5))
 @pytest.mark.parametrize("family", elements)
@@ -74,8 +101,13 @@ def test_create_element(cell, degree, family, variant):
             raise e
 
     try:
-        element = basix.create_element(family, cell, degree, *variant, True)
+        element = basix.create_element(family, cell, degree, *variant, discontinuous=True)
         assert element.degree == degree
     except RuntimeError as e:
         if len(e.args) == 0 or "dgesv" in e.args[0]:
             raise e
+
+
+def test_create_high_degree_lagrange():
+    basix.create_element(
+        basix.ElementFamily.P, basix.CellType.hexahedron, 7, basix.LagrangeVariant.gll_isaac)
