@@ -2,11 +2,12 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
 import numpy
 import pytest
 import sympy
 
+import basix
+
 
 def sympy_lagrange(celltype, n):
     # These basis functions were computed using symfem. They can be recomputed
@@ -186,7 +187,7 @@ def sympy_lagrange(celltype, n):
 
 
 def test_point():
-    lagrange = basix.create_element(basix.ElementFamily.P, basix.CellType.point, 0, True)
+    lagrange = basix.create_element(basix.ElementFamily.P, basix.CellType.point, 0, discontinuous=True)
     assert numpy.allclose(lagrange.tabulate(0, numpy.array([[]])), [[[1]]])
     assert numpy.allclose(lagrange.tabulate(0, numpy.array([[], []])), [[[1, 1]]])
 
@@ -291,7 +292,7 @@ def test_lagrange(celltype, degree):
                                     basix.LagrangeVariant.equispaced)
     pts = basix.create_lattice(celltype[0], 6, basix.LatticeType.equispaced, True)
     w = lagrange.tabulate(0, pts)[0]
-    assert(numpy.isclose(numpy.sum(w, axis=1), 1.0).all())
+    assert numpy.isclose(numpy.sum(w, axis=1), 1.0).all()
 
 
 @pytest.mark.parametrize("degree", [1, 2, 3, 4])
@@ -389,7 +390,7 @@ def test_celltypes(degree, celltype):
     pts = basix.create_lattice(celltype, 5,
                                basix.LatticeType.equispaced, True)
     w = tp.tabulate(0, pts)[0]
-    assert(numpy.allclose(numpy.sum(w, axis=1), 1.0))
+    assert numpy.allclose(numpy.sum(w, axis=1), 1.0)
 
 
 def leq(a, b):
@@ -413,20 +414,26 @@ def in_cell(celltype, p):
 
 @pytest.mark.parametrize("variant", [
     basix.LagrangeVariant.equispaced,
-    basix.LagrangeVariant.gll_warped, basix.LagrangeVariant.gll_isaac, basix.LagrangeVariant.gll_centroid,
-    basix.LagrangeVariant.chebyshev_warped, basix.LagrangeVariant.chebyshev_isaac,
+    basix.LagrangeVariant.gll_warped,
+    basix.LagrangeVariant.gll_isaac,
+    basix.LagrangeVariant.gll_centroid,
+    basix.LagrangeVariant.chebyshev_warped,
+    basix.LagrangeVariant.chebyshev_isaac,
     basix.LagrangeVariant.chebyshev_centroid,
-    basix.LagrangeVariant.gl_warped, basix.LagrangeVariant.gl_isaac, basix.LagrangeVariant.gl_centroid,
+    basix.LagrangeVariant.gl_warped,
+    basix.LagrangeVariant.gl_isaac,
+    basix.LagrangeVariant.gl_centroid,
     basix.LagrangeVariant.vtk
 ])
 @pytest.mark.parametrize("celltype", [
-    basix.CellType.triangle, basix.CellType.tetrahedron,
-    basix.CellType.quadrilateral, basix.CellType.hexahedron,
+    basix.CellType.triangle,
+    basix.CellType.tetrahedron,
+    basix.CellType.quadrilateral,
+    basix.CellType.hexahedron,
 ])
 @pytest.mark.parametrize("degree", range(1, 5))
 def test_variant_points(celltype, degree, variant):
-    e = basix.create_element(basix.ElementFamily.P, celltype, degree, variant, True)
-
+    e = basix.create_element(basix.ElementFamily.P, celltype, degree, variant, discontinuous=True)
     for p in e.points:
         assert in_cell(celltype, p)
 
@@ -441,10 +448,10 @@ def test_variant_points(celltype, degree, variant):
     basix.CellType.quadrilateral, basix.CellType.hexahedron,
 ])
 def test_continuous_lagrange(celltype, variant):
-    # The variants used in this test can only be used for discontinuous Lagrange,
-    # so trying to create them should throw a runtime error
+    # The variants used in this test can only be used for discontinuous
+    # Lagrange, so trying to create them should throw a runtime error
     with pytest.raises(RuntimeError):
-        basix.create_element(basix.ElementFamily.P, celltype, 4, variant, False)
+        basix.create_element(basix.ElementFamily.P, celltype, 4, variant, discontinuous=False)
 
 
 @pytest.mark.parametrize("celltype", [
@@ -456,8 +463,10 @@ def test_vtk_element(celltype, degree):
     if degree > 5 and celltype == basix.CellType.hexahedron:
         pytest.skip("Skipping slow test on hexahedron")
 
-    equi = basix.create_element(basix.ElementFamily.P, celltype, degree, basix.LagrangeVariant.equispaced, True)
-    vtk = basix.create_element(basix.ElementFamily.P, celltype, degree, basix.LagrangeVariant.vtk, True)
+    equi = basix.create_element(basix.ElementFamily.P, celltype, degree,
+                                basix.LagrangeVariant.equispaced, discontinuous=True)
+    vtk = basix.create_element(basix.ElementFamily.P, celltype, degree,
+                               basix.LagrangeVariant.vtk, discontinuous=True)
 
     assert vtk.points.shape == equi.points.shape
 
@@ -540,7 +549,7 @@ def test_vtk_element(celltype, degree):
 ])
 @pytest.mark.parametrize("degree", range(1, 5))
 def test_legendre(celltype, degree, variant):
-    e = basix.create_element(basix.ElementFamily.P, celltype, degree, variant, True)
+    e = basix.create_element(basix.ElementFamily.P, celltype, degree, variant, discontinuous=True)
     for p in e.points:
         assert in_cell(celltype, p)
 
@@ -558,7 +567,8 @@ def test_legendre(celltype, degree, variant):
 ])
 @pytest.mark.parametrize("degree", range(5))
 def test_dpc(celltype, degree, variant):
-    e = basix.create_element(basix.ElementFamily.DPC, celltype, degree, variant, True)
+    e = basix.create_element(basix.ElementFamily.DPC, celltype, degree,
+                             dpc_variant=variant, discontinuous=True)
     for p in e.points:
         assert in_cell(celltype, p)
 
@@ -571,7 +581,7 @@ def test_dpc(celltype, degree, variant):
 @pytest.mark.parametrize("degree", range(1, 5))
 def test_legendre_lagrange_variant(celltype, degree):
     e = basix.create_element(
-        basix.ElementFamily.P, celltype, degree, basix.LagrangeVariant.legendre, True)
+        basix.ElementFamily.P, celltype, degree, basix.LagrangeVariant.legendre, discontinuous=True)
 
     # Test that the basis functions are orthogonal
     pts, wts = basix.make_quadrature(celltype, degree * 2)
@@ -643,7 +653,7 @@ def test_legendre_lagrange_variant(celltype, degree):
 @pytest.mark.parametrize("degree", range(1, 5))
 def test_legendre_dpc_variant(celltype, degree):
     e = basix.create_element(
-        basix.ElementFamily.DPC, celltype, degree, basix.DPCVariant.legendre, True)
+        basix.ElementFamily.DPC, celltype, degree, dpc_variant=basix.DPCVariant.legendre, discontinuous=True)
 
     # Test that the basis functions are orthogonal
     pts, wts = basix.make_quadrature(celltype, degree * 2)
