@@ -2,10 +2,11 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
 import numpy as np
 import pytest
 
+import basix
+
 
 @pytest.mark.parametrize("order", [1, 2, 3])
 def test_quad(order):
@@ -16,14 +17,14 @@ def test_quad(order):
         for q, v in zip(Lpts, Lwts):
             Qpts.append([p[0], q[0]])
             Qwts.append(u * v)
-    basis = basix._basixcpp.tabulate_polynomial_set(basix.CellType.quadrilateral,
-                                                    order, 0, Qpts)[0]
-    ndofs = basis.shape[1]
+    basis = basix._basixcpp.tabulate_polynomial_set(
+        basix.CellType.quadrilateral, basix.PolysetType.standard, order, 0, Qpts)[0]
+    ndofs = basis.shape[0]
 
     mat = np.zeros((ndofs, ndofs))
     for i in range(ndofs):
         for j in range(ndofs):
-            mat[i, j] = sum(basis[:, i] * basis[:, j] * Qwts)
+            mat[i, j] = sum(basis[i, :] * basis[j, :] * Qwts)
 
     assert np.allclose(mat, np.eye(ndofs))
 
@@ -39,14 +40,14 @@ def test_pyramid(order):
                 sc = (1.0 - r[0])
                 Qpts.append([p[0] * sc, q[0] * sc, r[0]])
                 Qwts.append(u * v * sc * sc * w)
-    basis = basix._basixcpp.tabulate_polynomial_set(basix.CellType.pyramid,
-                                                    order, 0, Qpts)[0]
-    ndofs = basis.shape[1]
+    basis = basix._basixcpp.tabulate_polynomial_set(
+        basix.CellType.pyramid, basix.PolysetType.standard,  order, 0, Qpts)[0]
+    ndofs = basis.shape[0]
 
     mat = np.zeros((ndofs, ndofs))
     for i in range(ndofs):
         for j in range(ndofs):
-            mat[i, j] = sum(basis[:, i] * basis[:, j] * Qwts)
+            mat[i, j] = sum(basis[i, :] * basis[j, :] * Qwts)
 
     assert np.allclose(mat, np.eye(ndofs))
 
@@ -61,14 +62,14 @@ def test_hex(order):
             for r, w in zip(Lpts, Lwts):
                 Qpts.append([p[0], q[0], r[0]])
                 Qwts.append(u * v * w)
-    basis = basix._basixcpp.tabulate_polynomial_set(basix.CellType.hexahedron,
-                                                    order, 0, Qpts)[0]
-    ndofs = basis.shape[1]
+    basis = basix._basixcpp.tabulate_polynomial_set(
+        basix.CellType.hexahedron, basix.PolysetType.standard, order, 0, Qpts)[0]
+    ndofs = basis.shape[0]
 
     mat = np.zeros((ndofs, ndofs))
     for i in range(ndofs):
         for j in range(ndofs):
-            mat[i, j] = sum(basis[:, i] * basis[:, j] * Qwts)
+            mat[i, j] = sum(basis[i, :] * basis[j, :] * Qwts)
 
     assert np.allclose(mat, np.eye(ndofs))
 
@@ -83,14 +84,14 @@ def test_prism(order):
         for q, v in zip(Lpts, Lwts):
             Qpts.append([p[0], p[1], q[0]])
             Qwts.append(u * v)
-    basis = basix._basixcpp.tabulate_polynomial_set(basix.CellType.prism,
-                                                    order, 0, Qpts)[0]
-    ndofs = basis.shape[1]
+    basis = basix._basixcpp.tabulate_polynomial_set(
+        basix.CellType.prism, basix.PolysetType.standard, order, 0, Qpts)[0]
+    ndofs = basis.shape[0]
 
     mat = np.zeros((ndofs, ndofs))
     for i in range(ndofs):
         for j in range(ndofs):
-            mat[i, j] = sum(basis[:, i] * basis[:, j] * Qwts)
+            mat[i, j] = sum(basis[i, :] * basis[j, :] * Qwts)
 
     assert np.allclose(mat, np.eye(ndofs))
 
@@ -104,14 +105,41 @@ def test_prism(order):
     basix.CellType.prism,
 ])
 @pytest.mark.parametrize("order", [0, 1, 2, 3, 4])
-def test_cell(cell_type, order):
+def test_standard(cell_type, order):
     Qpts, Qwts = basix.make_quadrature(cell_type, 2*order + 1)
-    basis = basix._basixcpp.tabulate_polynomial_set(cell_type, order, 0, Qpts)[0]
+    basis = basix._basixcpp.tabulate_polynomial_set(
+        cell_type, basix.PolysetType.standard, order, 0, Qpts)[0]
 
-    ndofs = basis.shape[1]
+    ndofs = basis.shape[0]
     mat = np.zeros((ndofs, ndofs))
     for i in range(ndofs):
         for j in range(ndofs):
-            mat[i, j] = sum(basis[:, i] * basis[:, j] * Qwts)
+            mat[i, j] = sum(basis[i, :] * basis[j, :] * Qwts)
 
     assert np.allclose(mat, np.eye(ndofs))
+
+
+@pytest.mark.parametrize("cell_type", [
+    basix.CellType.interval,
+    basix.CellType.triangle,
+    basix.CellType.tetrahedron,
+    basix.CellType.quadrilateral,
+    basix.CellType.hexahedron,
+])
+@pytest.mark.parametrize("order", [0, 1, 2, 3, 4])
+def test_macroedge(cell_type, order):
+    if cell_type == basix.CellType.triangle and order > 2:
+        pytest.xfail("Degree > 2 edge macro polysets not implemented on triangles.")
+    if cell_type == basix.CellType.tetrahedron and order > 1:
+        pytest.xfail("Degree > 1 edge macro polysets not implemented on tetrahedra.")
+
+    Qpts, Qwts = basix.make_quadrature(cell_type, 2*order + 1, polyset_type=basix.PolysetType.macroedge)
+    basis = basix._basixcpp.tabulate_polynomial_set(
+        cell_type, basix.PolysetType.macroedge, order, 0, Qpts)[0]
+
+    ndofs = basis.shape[0]
+    mat = np.zeros((ndofs, ndofs))
+    for i in range(ndofs):
+        for j in range(ndofs):
+            mat[i, j] = sum(basis[i, :] * basis[j, :] * Qwts)
+    assert np.allclose(mat, np.eye(ndofs))
