@@ -2,11 +2,12 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
 import numpy as np
 import pytest
 import sympy
 
+import basix
+
 
 @pytest.mark.parametrize("celltype", [(basix.CellType.quadrilateral, 1.0),
                                       (basix.CellType.hexahedron, 1.0),
@@ -17,26 +18,26 @@ import sympy
 @pytest.mark.parametrize("order", range(9))
 def test_cell_quadrature(celltype, order):
     Qpts, Qwts = basix.make_quadrature(celltype[0], order)
-    assert(np.isclose(sum(Qwts), celltype[1]))
+    assert np.isclose(sum(Qwts), celltype[1])
 
 
 @pytest.mark.parametrize("m", range(7))
 @pytest.mark.parametrize("scheme", [basix.QuadratureType.Default, basix.QuadratureType.gll])
 def test_qorder_line(m, scheme):
-    Qpts, Qwts = basix.make_quadrature(scheme, basix.CellType.interval, m)
+    Qpts, Qwts = basix.make_quadrature(basix.CellType.interval, m, rule=scheme)
     x = sympy.Symbol('x')
     f = x**m
     q = sympy.integrate(f, (x, 0, (1)))
     s = 0.0
     for (pt, wt) in zip(Qpts, Qwts):
         s += wt * f.subs([(x, pt[0])])
-    assert(np.isclose(float(q), float(s)))
+    assert np.isclose(float(q), float(s))
 
 
 @pytest.mark.parametrize("m", range(6))
 @pytest.mark.parametrize("scheme", [basix.QuadratureType.Default, basix.QuadratureType.gauss_jacobi])
 def test_qorder_tri(m, scheme):
-    Qpts, Qwts = basix.make_quadrature(scheme, basix.CellType.triangle, m)
+    Qpts, Qwts = basix.make_quadrature(basix.CellType.triangle, m, rule=scheme)
     x = sympy.Symbol('x')
     y = sympy.Symbol('y')
     f = x**m + y**m
@@ -44,13 +45,13 @@ def test_qorder_tri(m, scheme):
     s = 0.0
     for (pt, wt) in zip(Qpts, Qwts):
         s += wt * f.subs([(x, pt[0]), (y, pt[1])])
-    assert(np.isclose(float(q), float(s)))
+    assert np.isclose(float(q), float(s))
 
 
 @pytest.mark.parametrize("m", range(1, 20))
 @pytest.mark.parametrize("scheme", [basix.QuadratureType.xiao_gimbutas])
 def test_xiao_gimbutas_tri(m, scheme):
-    Qpts, Qwts = basix.make_quadrature(scheme, basix.CellType.triangle, m)
+    Qpts, Qwts = basix.make_quadrature(basix.CellType.triangle, m, rule=scheme)
     x = sympy.Symbol('x')
     y = sympy.Symbol('y')
     f = x**m + 2 * y**m
@@ -58,13 +59,13 @@ def test_xiao_gimbutas_tri(m, scheme):
     s = 0.0
     for (pt, wt) in zip(Qpts, Qwts):
         s += wt * f.subs([(x, pt[0]), (y, pt[1])])
-    assert(np.isclose(float(q), float(s)))
+    assert np.isclose(float(q), float(s))
 
 
 @pytest.mark.parametrize("m", range(1, 16))
 @pytest.mark.parametrize("scheme", [basix.QuadratureType.xiao_gimbutas])
 def test_xiao_gimbutas_tet(m, scheme):
-    Qpts, Qwts = basix.make_quadrature(scheme, basix.CellType.tetrahedron, m)
+    Qpts, Qwts = basix.make_quadrature(basix.CellType.tetrahedron, m, rule=scheme)
     x = sympy.Symbol('x')
     y = sympy.Symbol('y')
     z = sympy.Symbol('z')
@@ -73,13 +74,13 @@ def test_xiao_gimbutas_tet(m, scheme):
     s = 0.0
     for (pt, wt) in zip(Qpts, Qwts):
         s += wt * f.subs([(x, pt[0]), (y, pt[1]), (z, pt[2])])
-    assert(np.isclose(float(q), float(s)))
+    assert np.isclose(float(q), float(s))
 
 
 @pytest.mark.parametrize("m", range(9))
 @pytest.mark.parametrize("scheme", [basix.QuadratureType.Default, basix.QuadratureType.gauss_jacobi])
 def test_qorder_tet(m, scheme):
-    Qpts, Qwts = basix.make_quadrature(scheme, basix.CellType.tetrahedron, m)
+    Qpts, Qwts = basix.make_quadrature(basix.CellType.tetrahedron, m, rule=scheme)
     x = sympy.Symbol('x')
     y = sympy.Symbol('y')
     z = sympy.Symbol('z')
@@ -88,7 +89,7 @@ def test_qorder_tet(m, scheme):
     s = 0.0
     for (pt, wt) in zip(Qpts, Qwts):
         s += wt * f.subs([(x, pt[0]), (y, pt[1]), (z, pt[2])])
-    assert(np.isclose(float(q), float(s)))
+    assert np.isclose(float(q), float(s))
 
 
 def test_quadrature_function():
@@ -109,7 +110,8 @@ def test_gll():
     m = 5
 
     # 1D interval
-    pts, wts = basix.make_quadrature(basix.QuadratureType.gll, basix.CellType.interval, m+1)
+    pts, wts = basix.make_quadrature(
+        basix.CellType.interval, m+1, rule=basix.QuadratureType.gll)
     pts, wts = 2*pts.flatten()-1, 2*wts.flatten()
     ref_pts = np.array([-1., 1., -np.sqrt(3/7), 0.0, np.sqrt(3/7)])
     assert (np.allclose(pts.flatten(), ref_pts))
@@ -119,7 +121,8 @@ def test_gll():
     assert np.isclose(sum(wts), 2)
 
     # 2D quad
-    pts, wts = basix.make_quadrature(basix.QuadratureType.gll, basix.CellType.quadrilateral, m+1)
+    pts, wts = basix.make_quadrature(
+        basix.CellType.quadrilateral, m+1, rule=basix.QuadratureType.gll)
     pts, wts = 2*pts-1, 4*wts
     ref_pts2 = np.array([[x, y] for x in ref_pts for y in ref_pts])
     assert (np.allclose(pts, ref_pts2))
@@ -129,7 +132,8 @@ def test_gll():
     assert np.isclose(sum(wts), 4)
 
     # 3D hex
-    pts, wts = basix.make_quadrature(basix.QuadratureType.gll, basix.CellType.hexahedron, m+1)
+    pts, wts = basix.make_quadrature(
+        basix.CellType.hexahedron, m+1, rule=basix.QuadratureType.gll)
     pts, wts = 2*pts-1, 8*wts
     ref_pts3 = np.array([[x, y, z] for x in ref_pts for y in ref_pts for z in ref_pts])
     assert (np.allclose(pts, ref_pts3))
