@@ -2,11 +2,12 @@
 # FEniCS Project
 # SPDX-License-Identifier: MIT
 
-import basix
 import numpy
 import pytest
 import sympy
 
+import basix
+
 
 def sympy_rt(celltype, n):
     # These basis functions were computed using symfem. They can be recomputed
@@ -134,7 +135,7 @@ def test_tri(degree):
                     for k, p in enumerate(pts):
                         wsym[k, i, j] = wd.subs([(x, p[0]), (y, p[1])])
 
-            assert(numpy.isclose(wtab[basix.index(kx, ky)], wsym).all())
+            assert numpy.isclose(wtab[basix.index(kx, ky)], wsym).all()
 
 
 @pytest.mark.parametrize("degree", [1, 2, 3])
@@ -161,4 +162,4 @@ def test_tet(degree):
                         for k, p in enumerate(pts):
                             wsym[k, i, j] = wd.subs([(x, p[0]), (y, p[1]), (z, p[2])])
 
-                assert(numpy.isclose(wtab[basix.index(kx, ky, kz)], wsym).all())
+                assert numpy.isclose(wtab[basix.index(kx, ky, kz)], wsym).all()
